/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2007 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.plugins.paye;

import fr.gouv.finances.dgfip.xemelios.common.config.ListeResultatModel;
import fr.gouv.finances.dgfip.xemelios.data.DataLayerManager;
import fr.gouv.finances.dgfip.xemelios.data.DataResultSet;
import fr.gouv.finances.cp.xemelios.data.impl.MySqlDataLayer;
import fr.gouv.finances.cp.xemelios.data.impl.mysql.MySqlDataResultSet;
import fr.gouv.finances.dgfip.xemelios.data.impl.sqlconfig.TDocument;
import fr.gouv.finances.dgfip.xemelios.data.impl.sqlconfig.TEtat;

import fr.gouv.finances.dgfip.xemelios.auth.XemeliosUser;
import fr.gouv.finances.dgfip.xemelios.plugins.SearchPlugin;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import org.apache.log4j.Logger;

/**
 *
 * @author chm
 */
public class SommeHeuresSup extends SearchPlugin {
    private static final Logger logger = Logger.getLogger(SommeHeuresSup.class);
    private String opNbHeures,start,end, nbHeures;
    private String mtNet, op_mtNet;
    private String mtBrut, op_mtBrut;
    private String agenStatut, op_agenStatut;
    private String emploi, op_emploi;
    private String service, op_service;
    private String grade, op_grade;
    private String indice, op_indice;
    
    /** Creates a new instance of SommeHeuresSup */
    public SommeHeuresSup(XemeliosUser user) {
        super(user);
    }

    public void computeParameters() throws Exception {
        nbHeures = getParameters().get("VALEUR:nbHeures");
        opNbHeures = getParameters().get("OPERATEUR:nbHeures");
        String yearStart = getParameters().get("VALEUR:yearStart");
        String monthStart = getParameters().get("VALEUR:monthStart");
        String yearEnd = getParameters().get("VALEUR:yearEnd");
        String monthEnd = getParameters().get("VALEUR:monthEnd");
        
        start = yearStart+monthStart;
        end = yearEnd+monthEnd;
        mtNet = getParameters().get("VALEUR:mtnet");
        op_mtNet = getParameters().get("OPERATEUR:mtnet");
        mtBrut = getParameters().get("VALEUR:mtbrut");
        op_mtBrut = getParameters().get("OPERATEUR:mtbrut");
        agenStatut = getParameters().get("VALEUR:agent_statut");
        op_agenStatut = "xem:StringEquals";
        emploi = getParameters().get("VALEUR:emploi");
        op_emploi = getParameters().get("OPERATEUR:emploi");
        service = getParameters().get("VALEUR:service");
        op_service = getParameters().get("OPERATEUR:service");
        grade = getParameters().get("VALEUR:grade");
        op_grade = getParameters().get("OPERATEUR:grade");
        indice = getParameters().get("VALEUR:indice");
        op_indice = getParameters().get("OPERATEUR:indice");        
        StringBuffer error = new StringBuffer();
        boolean hasError = false;
        try {
        	if (mtNet != null && !"".equals(mtNet)) {
        		new BigDecimal (mtNet);
        	}
        } catch (NumberFormatException nfe) {
        	error.append("Le champ montant net ne contient pas un montant valide\n");
        	hasError = true;
        }
        try {
        	if (mtBrut != null && !"".equals(mtBrut)) {
        		new BigDecimal (mtBrut);
        	}
        } catch (NumberFormatException nfe) {
        	error.append("Le champ montant brut ne contient pas un montant valide\n");
        	hasError = true;
        }
        try {
        	if (indice != null && !"".equals(indice)) {
        		new BigDecimal (indice);
        	}
        } catch (NumberFormatException nfe) {
        	error.append("Le champ montant indice ne contient pas un nombre valide\n");
        	hasError = true;
        }
        if(hasError) {
            logger.error(error.toString());
            throw new Exception(error.toString());
        }
    }

    @Override
    public DataResultSet getResultSet() throws Exception {
        computeParameters();
        StringBuilder request = new StringBuilder();
        request.append( "SELECT AGENT_MATRICULE, SUM(HEURES_SUPP) ");
        request.append ("FROM PAYE_ETATPAYE_INDEX_PAYE ");
        request.append ("WHERE ANNEE_MOIS>=? AND ANNEE_MOIS<=? ");
        /* MONTANT NET */
        if (mtNet!=null && !"".equals(mtNet)) {
        	request.append (" AND MT_NET ").append (op_mtNet).append (mtNet).append (" ");
        }
        /* MONTANT BRUT */
        if (mtBrut!=null && !"".equals(mtBrut)) {
        	request.append (" AND MT_BRUT ").append (op_mtBrut).append (mtBrut).append (" ");
        }
        /* Grade */
        if (grade!=null && !"".equals(grade)) {
        	request.append (" AND AGENT_GRADE ").append (convertOpString (op_grade, grade)).append (" ");
        }
        /* Statut */
        if (agenStatut!=null && !"".equals(agenStatut)) {
        	request.append (" AND AGENT_STATUT ").append (convertOpString (op_agenStatut, agenStatut)).append (" ");
        }
        /* Emploi */
        if (emploi!=null && !"".equals(emploi)) {
        	request.append (" AND AGENT_METIER ").append (convertOpString (op_emploi, emploi)).append (" ");
        }
        /* Service */
        if (service!=null && !"".equals(service)) {
        	request.append (" AND SERVICE ").append (convertOpString (op_service, service)).append (" ");
        }
        /* Indice */
        if (indice!=null && !"".equals(indice)) {
        	request.append (" AND AGENT_INDICE ").append (op_indice).append (indice).append (" ");
        }
        
        request.append ("GROUP BY AGENT_MATRICULE HAVING SUM(HEURES_SUPP)").append(opNbHeures).append(nbHeures);
        Connection con = null;
        PreparedStatement ps = null;
        MySqlDataLayer dl = (MySqlDataLayer)DataLayerManager.getImplementation();
        TDocument pc = dl.getPersistenceConfig(getEtatModel().getParent(),getUser()).getLayer("mysql").getDocument(getEtatModel().getParent().getId());
        TEtat sec = pc.getEtat(getEtatModel().getId());
        String tableName = sec.getIndexTable("T1").getBaseName();
        try {
            con = dl.getConnection();
logger.debug(request.toString());
            ps = con.prepareStatement(request.toString());
            ps.setString(1,start);
            ps.setString(2,end);
            ResultSet rs = ps.executeQuery();
            ArrayList<String> matricules = new ArrayList<String>();
            while(rs.next()) {
                String matr = rs.getString(1);
                matricules.add(matr);
            }
            if(matricules.size()>0) {
	            request = new StringBuilder();
	            request.append("SELECT SQL_CALC_FOUND_ROWS DOC_ID, AGENT_MATRICULE FROM ").append(tableName);
	            request.append(" WHERE ANNEE_MOIS>=").append(start).append(" AND ANNEE_MOIS<=").append(end).append(" AND AGENT_MATRICULE IN (");
	            for(String s:matricules) request.append("'").append(s).append("',");
	            request.deleteCharAt(request.length()-1);
	            request.append(") AND COLLECTIVITE='").append(getCollectivite().key).append("' AND BUDGET='").append(getBudget().key).append("' ");
                    ListeResultatModel lrm = getListeResultat();
	            return new MySqlDataResultSet(getElementModel(),getCollectivite(),getBudget(),"/n:DocumentPaye/n:DonneesIndiv/n:PayeIndivMensuel",request.toString(),lrm,dl,getUser());
            }
        } catch(SQLException sqlEx) {
            sqlEx.printStackTrace();
        } finally {
            if(con!=null) dl.releaseConnection(con);
        }
        return DataLayerManager.getImplementation().getEmptyDataResultSet();
    }
    
}
