/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.controls.models;

import org.apache.commons.lang.StringEscapeUtils;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;
import javax.xml.namespace.QName;

/**
 * Modelise une entree de liste de selection
 * @author chm
 */
public class OptionModel implements XmlMarshallable {
	public static final transient String TAG = "option";
        public static final transient QName QN = new QName(TAG);
	private String value;
	private StringBuffer libelle;

	public OptionModel(QName tagName) {
            super();
            libelle = new StringBuffer();
        }
        public OptionModel(String value, String libelle) {
            this(QN);
            this.value=value;
            this.libelle.append(libelle);
        }
	public void addCharacterData(String cData) throws SAXException {
		libelle.append(cData);
	}
	public void addChild(XmlMarshallable child, QName tagName)throws SAXException {}
	public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
		value = attributes.getValue("value");
		return this;
	}
	public void marshall(XmlOutputter output) {
	    output.startTag(TAG);
	    output.addAttribute("value",StringEscapeUtils.escapeXml(value));
	    output.addCharacterData(StringEscapeUtils.escapeXml(libelle.toString()));
	    output.endTag(TAG);
	}
	public void validate() throws InvalidXmlDefinition {}
	public String getLibelle() {return libelle.toString();}
	public String getValue() {return value;}
	@Override
	public OptionModel clone() {
	    OptionModel om = new OptionModel(QN);
	    om.value=this.value;
	    om.libelle = this.libelle;
	    return om;
	}

    public XmlMarshallable getChildToModify(String uri, String localName, String qName, Attributes atts) {
        return null;
    }

    public QName getQName() {
        return QN;
    }
}
