/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.plugins;

import fr.gouv.finances.cp.utils.ui.DisplayExceptionDlg;
import fr.gouv.finances.cp.xemelios.ui.search.ResultDisplayerThread;
import fr.gouv.finances.dgfip.xemelios.auth.XemeliosUser;
import fr.gouv.finances.dgfip.xemelios.common.config.ListeResultatModel;
import fr.gouv.finances.dgfip.xemelios.data.DataResultSet;
import fr.gouv.finances.dgfip.xemelios.plugins.SearchPlugin;
import javax.swing.SwingUtilities;

/**
 * Defines contract to make search plugins.
 * An implementation <b>must</b> provide a constructor with a <tt>XemeliosUser</tt> parameter
 * @author chm
 */
public class SearchPluginRunner extends XemeliosPlugin {
    private ListeResultatModel lrm;
    private XemeliosUser user;
    private SearchPlugin plugin;

    public SearchPluginRunner(XemeliosUser user) {
        super();
        this.user=user;
    }
    
    public void setPlugin(SearchPlugin plugin) {
        this.plugin = plugin;
    }
    /* (non-Javadoc)
     * @see org.jdesktop.swingworker.SwingWorker#doInBackground()
     */
    @Override
    public DataResultSet doInBackground() throws Exception {
        plugin.setParameters(getParameters());
        plugin.setElementModel(getElementModel());
        plugin.setListeResultatModel(lrm);
        return plugin.getResultSet();
    }


    public void startSearch() throws Exception {
        execute();
    }

    @Override
    public int getPluginType() {
        return XemeliosPlugin.TYPE_SEARCH;
    }

    public DataResultSet getResultSet() throws Exception {
        return (DataResultSet) get();
    }
    
    public void setListeResultat(ListeResultatModel lrm) { this.lrm = lrm; }
    public ListeResultatModel getListeResultat() { return lrm; }

    
    public XemeliosUser getUser() { return user; }

    @Override
    public void done() {
        try {
            DataResultSet rs = (DataResultSet)get();
            Runnable code = new ResultDisplayerThread(rs,getListeResultat(),getStart(),getElementModel(),getSearchWindow());
            if(SwingUtilities.isEventDispatchThread()) code.run();
            else try { SwingUtilities.invokeAndWait(code); } catch(Throwable t) {}
            firePropertyChange("COMPLETED",Boolean.FALSE,Boolean.TRUE);
        } catch(Throwable t) {
            DisplayExceptionDlg.displayException(getOwner(),t);
        } finally {
            setProgressVisible(false);
            super.done();
        }
    }
}
