/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2009 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.dgfip.xemelios.web.navigate;

import fr.gouv.finances.dgfip.utils.NavigationContext;
import fr.gouv.finances.dgfip.xemelios.auth.XemeliosUser;
import fr.gouv.finances.dgfip.xemelios.common.config.Loader;
import fr.gouv.finances.dgfip.xemelios.common.config.customLinks.CustomLinkParameters;
import fr.gouv.finances.dgfip.xemelios.common.config.customLinks.CustomLinkResolver;
import fr.gouv.finances.dgfip.xemelios.common.config.customLinks.UnresolvedException;
import fr.gouv.finances.dgfip.xemelios.web.AttributeHelper;
import fr.gouv.finances.dgfip.xemelios.web.utils.StdDispatchAction;
import java.util.Enumeration;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.transform.stream.StreamResult;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 *
 * @author chm
 */
public class CustomLinkAction extends StdDispatchAction {
    private static final Logger logger = Logger.getLogger(CustomLinkAction.class);

    @Override
    protected ActionForward unspecified(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
        logger.debug(request.getQueryString());
        CustomLinkParameters parameters = new CustomLinkParameters();
        NavigationContext ctx = new NavigationContext();
        XemeliosUser user = AttributeHelper.getUser(request.getSession());
        String[] xslParamsArray = request.getParameterValues("xsl:param");

        for(Enumeration enumer = request.getParameterNames();enumer.hasMoreElements();) {
            String paramName = (String)enumer.nextElement();
            String paramValue = request.getParameter(paramName);
            if("srcDocId".equals(paramName)) parameters.setSrcDocId(paramValue);
            else if("srcEtatId".equals(paramName)) parameters.setSrcEtatId(paramValue);
            else if("srcElementId".equals(paramName)) parameters.setSrcElementId(paramValue);
            else if("srcCollectivite".equals(paramName)) parameters.setSrcCollectivite(paramValue);
            else if("srcBudget".equals(paramName)) parameters.setSrcBudget(paramValue);
            else {
                parameters.addParameter(paramName, paramValue);
            }
        }
        try {
            String clrClassName = parameters.getCustomLinkResolverName(Loader.getDocumentsInfos(null));
            Class clazz = Class.forName(clrClassName);
            CustomLinkResolver resolver = (CustomLinkResolver)clazz.newInstance();
            ctx = resolver.resolve(parameters, new StreamResult(response.getOutputStream()), ctx, user);

        } catch(UnresolvedException ex) {
//            request.setAttribute("error.message",ex.getMessage());
//            return mapping.findForward("simple.error");
            String url = AttributeHelper.getPreviousView(request.getSession()).concat("&xsl:param=(error.message,").concat(ex.getMessage()).concat(")");
            return new ActionForward(url);
        } catch(Exception ex) {
            throw ex;
        }
        AttributeHelper.setPreviousView(request.getSession(), request.getServletPath()+"?"+request.getQueryString());
        return null;
    }

}
